version 11
clear 
set more off
capture log close

use cps1re74.dta, clear

* dummies for unemployed
generate u74=re74==0
generate u75=re75==0

generate age3=age^3
gen edre74=ed*re74
************************************************************************
*** Propensity Score Matching   without replacement, random                                     ***
************************************************************************

* Now we try to construct a control group out of the CPS

* PSCORE = likelihood to select into the treatment
gen ed2=ed^2


probit treat age age2 age3 ed black hisp married nodeg re74 re75 edre74 u74 u75 ed2
predict ps, pr



bys treat: summarize ps
* the mean prob of treatment is much smaller in the CPS


* number of treated individuals
count if treat==1
scalar treated=r(N)
di treated

* first the treated
gsort -treat 
generate id=_n

* some variables we need in NN-Match
generate diff=.
generate match=0
generate matched=0
generate number=0
generate Km=0
generate re78_0=0

local delta=0.0001
* a loop over all treated individuals
* without replacement 
local j=1
while `j'<=treated {
	* calculate diff +between treated individual j and all nontreated
    quietly replace diff=abs(ps[`j']-ps) if treat==0 
	* set of matches within +/- delta
	quietly replace match=1 if diff<=`delta'
	quietly count if match==1
	quietly replace number=r(N) if _n==`j'
	* mark which observation is used (K_M)
	quietly replace Km=Km+1/r(N) if match==1
	* predicted counterfactual as the mean fo the matching partners (here only one)
	quietly sum re78 if match==1
	quietly replace matched=1 if _n==`j' & r(N)>0
	quietly replace re78_0=r(mean) if _n==`j'
	* cleaning up for the next treated
	quietly replace match=0
	quietly replace diff=.
    local j=`j'+1
}
* How many controls are used in matching?
count if Km>0
count if matched==1
* average predicted re78 without treatment
summarize re78_0 if treat==1 & matched==1
scalar Y0_nnmatch=r(mean)
summarize re78 if treat==1 & matched==1
scalar Y1bar=r(mean)
scalar ATOT_nnmatch=Y1bar-Y0_nnmatch
display ATOT_nnmatch

* for control
*nnmatch re78 treat ps, tc(att)

* regression estimator, controls are weighted with number of times it is matched
generate weights=Km
replace weights=weights+1 if treat==1 & matched==1
regress  re78 treat  age ed black hisp married nodeg re74 re75 u74 u75 [iweight=weights]
scalar ATOT_nnweights=_b[treat]
