version 11
clear 
set more off
capture log close


**********************************************************************
*** Estimation of treatment effects with non-experimental data     ***
**********************************************************************

* let's assume, there is no experimental control group
* instead we use a large sample from the current population survey (CPS)

use cps1re74.dta, clear

* dummies for unemployed
generate u74=re74==0
generate u75=re75==0

tab treat
* here we have the 185 treated persons from the experiment (treatment group)
* and 15992 not treated from the CPS

* First: mean comparisson in the unmatched sample
tabstat age ed black hisp married nodeg re74 re75 u74 u75, statistics( mean ) by(treat) columns(variables) format(%9.4g)

* these are the values in the second row of table 2

* Naive estimation
summarize re78 if treat==0
scalar Y0_CPS=r(mean)
summarize re78 if treat==1
scalar Y1bar=r(mean)
scalar ATOT_CPSnaive=Y1bar-Y0_CPS
display ATOT_CPSnaive

* difference of means in unmatched sample
* it is negative!

* bootstraping to estimate the standard error
*reg re78 treat, vce(bootstrap, reps(500))

* regression based estimator
reg  re78 treat  age ed black hisp married nodeg re74 re75 u74 u75

* coeff of treat is the last value in row 2
* closer to our benchmark but still to small

* may be we can improve this with a more flexible functional form
generate age3=age^3
gen edre74=ed*re74

reg  re78 treat  age ed black hisp married nodeg re74 re75 u74 u75 age2 age3 edre74
* we are getting closer to the benchmark and it is significant 

************************************************************************
*** Propensity Score Matching                                        ***
************************************************************************

* Now we try to construct a control group out of the CPS

* PSCORE = likelihood to select into the treatment
gen ed2=ed^2


probit treat age age2 age3 ed black hisp married nodeg re74 re75 edre74 u74 u75 ed2
predict ps, pr



bys treat: summarize ps
* the mean prob of treatment is much smaller in the CPS


* number of treated individuals
count if treat==1
scalar treated=r(N)
di treated
* first the treated
gsort -treat

* some variables we need in NN-Match
generate diff=.
generate match=0
generate number=0
generate Km=0
generate re78_0=0

* a loop over all treated individuals
local j=1
while `j'<=treated {
	* calculate diff between treated individual j and all nontreated
    quietly replace diff=abs(ps[`j']-ps) if treat==0
	* calculate the min difference (d_M)
	quietly egen mindiff=min(diff) 
	* set of matches with mindiff
	quietly replace match=1 if diff==mindiff
	* how many observations with mindiff? (#J_M)
	quietly count if match==1
	quietly replace number=r(N) if _n==`j'
	* count how offen a observation is used (K_M)
	quietly replace Km=Km+1/r(N) if match==1
	* predicted counterfactual as the mean fo the matching partners
	quietly sum re78 if match==1
	quietly replace re78_0=r(mean) if _n==`j'
	drop mindiff
	quietly replace match=0
    local j=`j'+1
}
* How many controls are used in matching?
count if Km>0

* average predicted re78 without treatment
summarize re78_0 if treat==1
scalar Y0_nnmatch=r(mean)
scalar ATOT_nnmatch=Y1bar-Y0_nnmatch
display ATOT_nnmatch
* that is far away from the result by Dehejia & Wahba
* but much closer to the benchmark 


* for control: nnmatch.ado
* let stata know where it is to find with
* adopath + "path"
*nnmatch re78 treat ps, tc(att)

* regression estimator, controls are weighted with weighted number of times it is matched
generate weights=Km
replace weights=weights+1 if treat==1
regress  re78 treat  age ed black hisp married nodeg re74 re75 u74 u75 [iweight=weights]
scalar ATOT_nnweights=_b[treat]

* real Nearest-Neighbor Matching (not on Pscore)
*nnmatch re78 treat  age ed black hisp married nodeg re74 re75 u74 u75, tc(att) m(1) metric(maha)

